from fastapi import APIRouter, Depends, HTTPException
from starlette import status

from api import get_conn
from model.bcd_request import BCDRequestModel, BCDProtectionRequirements, BCDSystemType
from model.wapi.cntl import APIToken, Mgr
from util.auth import check_auth

router = APIRouter(
    prefix="/api/bcd_request",
    tags=["bcd_request"]
)
js_mods = ['bcdrequest']


# TODO: implement methods...
@router.post('')
async def handle_request(bcd_request: BCDRequestModel, token: APIToken = Depends(check_auth), conn=Depends(get_conn)):
    user = Mgr.check_token(conn, token)

    if user:
        return "Success"

    raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR)


def validate_bcd_request(bcd_request: BCDRequestModel) -> bool:
    # TODO: Check if provided user is part of the specified group

    if (bcd_request.protection_requirement == BCDProtectionRequirements.SPECIAL and
            bcd_request.protection_requirement_note is None):
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail='A description of the special protection requirement is required.'
        )
    elif (bcd_request.protection_requirement == BCDProtectionRequirements.SPECIAL and
          bcd_request.protection_requirement_note is not None and
          len(bcd_request.protection_requirement_note.strip()) == 0):
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail='A description of the special protection requirement is required.'
        )

    if bcd_request.type_of_system == BCDSystemType.OTHER and bcd_request.type_of_system_note is None:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail='A detailed description of the system type other must be provided.'
        )
    elif (bcd_request.type_of_system == BCDSystemType.OTHER and bcd_request.type_of_system_note is not None and
          len(bcd_request.type_of_system_note.strip()) == 0):
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail='A detailed description of the system type other must be provided.'
        )

    if bcd_request.type_of_system == BCDSystemType.SERVER and bcd_request.load_balancer is None:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail='You must provide load balancer info for server systems.'
        )

    if (bcd_request.type_of_system == BCDSystemType.SERVER and
            bcd_request.protection_requirement == BCDProtectionRequirements.LEVEL_2 and
            bcd_request.load_balancer is True):
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail='Load balancers are not supported with level two protection requirements.'
        )

    return True
